@@ -1,7 +1,7 @@
 """"""
 import os
 from pathlib import Path
-from .tools import load_json, eprint
+from .tools import load_json, eprint, strToValue
 from askGPT import DATA_PATH
 import toml
 from .api.openai import ChatGPT
@@ -22,6 +22,7 @@ basicConfig["retryMaxDelay"] = basicConfig.get("retryMaxDelay",60)
 basicConfig["retryMultiplier"] = basicConfig.get("retryMultiplier",2)
 basicConfig["verbose"] = basicConfig.get("verbose", False)
 basicConfig["debug"] = basicConfig.get("debug", False)
+basicConfig["updateScenarios"] = basicConfig.get("updateScenarios", True)
 
 class Config(object):
     def __init__(self):
@@ -43,7 +44,7 @@ class Config(object):
         self.update()
         self.chat = ChatGPT(self)
         self.chat.loadLicense()
-        self.version="0.4.18"
+        self.version="0.4.25"
         self.data_path = DATA_PATH
 
     def loadProgConfig(self):
@@ -54,15 +55,8 @@ class Config(object):
             self.saveConfig()
 
     def updateParameter(self,key, val):
-        if val == "true":
-            val = True
-        elif val == "false":
-            val = False
-        elif val.isnumeric():
-            val = int(val)
-        elif val.replace(".","",1).isnumeric():
-            val = float(val)
-        elif key in self.sessionConfig: # order matters
+        val = strToValue(val)
+        if key in self.sessionConfig: # order matters
             if self.sessionConfig[key] != val:
                 print(f"{key}] = {val}")
                 self.sessionConfig[key] = val
@@ -120,6 +114,7 @@ class Config(object):
         self.progConfig["retryMultiplier"] = self.progConfig.get("retryMultiplier",2)
         self.progConfig["verbose"] = self.progConfig.get("verbose", False)
         self.progConfig["debug"] = self.progConfig.get("debug", False)
+        self.progConfig["updateScenarios"] = self.progConfig.get("updateScenarios", True)
 
     def printConfig(self):
         """Print the configuration file"""
