@@ -8,7 +8,7 @@ __title__ = 'askGPT'
 __author__ = 'Meir Michanie'
 __license__ = 'MIT'
 __credits__ = ''
-__version__ = "0.4.18"
+__version__ = "0.4.25"
 
 import os
 from .api.openai import ChatGPT
@@ -74,10 +74,11 @@ def shell(config):
 @click.option("--retry-max-delay", default=basicConfig["retryMaxDelay"], type=float, help="max delay between retries")
 @click.option("--verbose/--quiet", default=False, help="Show verbose output")
 @click.option("--debug/--no-debug", default=False, help="Debug")
+@click.option("--updateScenarios/--keep-scenarios", default=True, help="Load Distribution scenarios")
 
 def config(config, user_prompt, ai_prompt, max_tokens,model, temperature, top_p, 
 frequency_penalty, presence_penalty, show_disclaimer,max_retries,
-retry_delay,retry_multiplier,retry_max_delay, verbose, debug):
+retry_delay,retry_multiplier,retry_max_delay, verbose, debug, updateScenarios):
     """
 Change config values"""
     config.progConfig["userPrompt"] = user_prompt
@@ -95,6 +96,7 @@ Change config values"""
     config.progConfig["retryMaxDelay"] = retry_max_delay
     config.progConfig["verbose"] = verbose
     config.progConfig["debug"] = debug
+    config.progConfig["updateScenarios"] = updateScenarios
 
 
     jsonConfig = {'name':'askGPT','default':config.progConfig}
@@ -113,6 +115,7 @@ def edit(config,subject,submit, scenario):
     """
 Edit a conversation"""
     subject = sanitizeName(subject)
+    scenario = sanitizeName(scenario)
     lines = list()
     if os.path.isfile(os.path.join(config.conversations_path, subject + config.fileExtention)):
         with open(os.path.join(config.conversations_path, subject + config.fileExtention), "r") as f:
@@ -137,6 +140,8 @@ Edit a conversation"""
 def train(config, subject, scenario, temperature):
     """Train the model with the conversation"""
     config.progConfig["temperature"] = temperature
+    subject = sanitizeName(subject)
+    scenario = sanitizeName(scenario)
     config.chat.submitDialog(subject, scenario)
 
 @cli.command()
@@ -147,6 +152,8 @@ def train(config, subject, scenario, temperature):
 
 def submit(config, subject, scenario, temperature):
     """Submit without editing the scenario file to openAi api and print out the response."""
+    subject = sanitizeName(subject)
+    scenario = sanitizeName(scenario)
     config.progConfig["temperature"] = temperature
     ai = config.chat.submitDialog(subject, scenario)
     print(ai)
@@ -161,6 +168,7 @@ def submit(config, subject, scenario, temperature):
 @click.argument("whatToShow", default="config")
 @click.argument("subject", default="" )
 def show(config, whattoshow, subject):
+    subject = sanitizeName(subject)
     """Show config|scenarios|subjects or the conversation inside a subject."""
     if subject == "":
         if whattoshow == "config":
@@ -211,6 +219,7 @@ Save the API keys to query OpenAI"""
 def delete(config, subject, all):
     """
 Delete the previous conversations saved by askGPT"""
+    subject = sanitizeName(subject)
     if all:
         for subject in config.get_list():
             os.remove(os.path.join(config.conversations_path, subject + config.fileExtention))
@@ -241,6 +250,8 @@ Delete the previous conversations saved by askGPT"""
 def query(config, subject, enquiry, scenario,model, temperature,max_tokens, top_p,  frequency_penalty, presence_penalty, verbose, save, retry, execute): 
     """
 Query the OpenAI API with the provided subject and enquiry"""
+    subject = sanitizeName(subject)
+    scenario = sanitizeName(scenario)
     enquiry = config.progConfig["userPrompt"] + enquiry
     ai = ""
     if subject:
@@ -280,8 +291,12 @@ Query the OpenAI API with the provided subject and enquiry"""
                     if str(e) == "openai.error.RateLimitError":
                         eprint("Error: Too many requests. We will try again")
                     eprint("Error: " + str(e))
-                    eprint(f"Retrying again in {sleepBetweenRetries} seconds...")
-                    time.sleep(sleepBetweenRetries)
+                    eprint(f"Retrying again in {sleepBetweenRetries} seconds.")
+                    for i in range(sleepBetweenRetries):
+                        time.sleep(1.0)
+                        eprint(".")
+                    eprint("\n")
+                    eprint("Retrying ...")
                     sleepBetweenRetries *= config.progConfig["retryMultiplier"] 
                     if sleepBetweenRetries > config.progConfig["retryMaxDelay"]:
                         sleepBetweenRetries = config.progConfig["retryMaxDelay"]
