@@ -1,5 +1,6 @@
 import cmd
-import shlex 
+import shlex
+import shutil 
 from .tools   import eprint, sanitizeName
 import toml
 import os
@@ -12,13 +13,18 @@ from rich.style import Style
 from rich.prompt import Prompt, Confirm
 from rich.panel import Panel
 from rich.markdown import Markdown
+from .tools import strToValue
+from rich.console import Console
+import click
+from filecmp import cmp
+import subprocess
 
 danger_style = Style(color="red", blink=False, bold=True)
 attention_style = Style(color="yellow", blink=False, bold=True)
 ok_style = Style(color="green", blink=False, bold=False)
-from rich.console import Console
 console = Console()
 
+
 """Here we will define the class Shell which is a child of cmd.cmd which will allow us to run all the commands interactively such as query, config, edit."""
 class Shell(cmd.Cmd):
     def __init__(self, config) -> None:
@@ -31,17 +37,38 @@ class Shell(cmd.Cmd):
         self.ruler = "-"
         self._config = config
         self.commands = dict()
+        self.lastResponse = None
         self._register_commands()
         self.conversation_parameters = {
             "subject": "test",
             "scenario": "Neutral",
             "model": "text-davinci-003",
-            "defaultCommand": "query"
+            "defaultCommand": "query",
+            "execute": False
 
         }
         if os.path.exists(os.path.join(self._config.settingsPath, "last.toml")):
-            self.conversation_parameters = toml.load(os.path.join(self._config.settingsPath, "last.toml"))
+            self.conversation_parameters.update(toml.load(os.path.join(self._config.settingsPath, "last.toml")))
         self.prompt = f"{self.conversation_parameters['scenario']}> "
+        self.do_update("")
+        
+
+    def do_update(self,args):
+        """replace the current scenarios files"""
+        args = shlex.split(args)
+        if self._config.progConfig["updateScenarios"] == False:
+            eprint("Update is disabled in the config file")
+            return
+        if not cmp(os.path.join(self._config.data_path, "scenarios.json"), os.path.join(self._config.settingsPath, "scenarios.json")):
+            if not Confirm.ask("New scenarios available.Would you like to replace the current ones?"):
+                eprint("Scenarios files matched. No need to overwrite.")
+                return 
+            if Confirm.ask("Would you like to make a backup of the current one?"):
+                shutil.copyfile(os.path.join(self._config.settingsPath, "scenarios.json"), os.path.join(self._config.settingsPath, "scenarios.json.bak"))
+            shutil.copyfile(os.path.join(self._config.data_path, "scenarios.json"), os.path.join(self._config.settingsPath, "scenarios.json"))
+            eprint("Scenarios updated, you need to restart to load the new scenario file")
+            return 
+            
 
     def do_clone(self,args):
         """if len(arg) == 1 then copy the current conversation to a new file using arg[1]"""
@@ -109,6 +136,10 @@ class Shell(cmd.Cmd):
         command = " ".join(args)
         os.system(command)
 
+    """Write function similar to do_exec but that it is able to save the output from the command being executed."""
+    def execStdout(command: list):
+        result = subprocess.run([command], stdout=subprocess.PIPE)
+        return result.stdout.decode('utf-8')
 
     def do_set(self,args):
         """Set conversation_parameters checking that the keys exist and that the values are valid"""
@@ -119,7 +150,7 @@ class Shell(cmd.Cmd):
             return
         if len(args) == 2:
             key = args[0]
-            val = args[1]
+            val = strToValue(args[1])
             if key in self.conversation_parameters:
                 if key == "scenario":
                     if val in self._config.scenarios:
@@ -138,15 +169,7 @@ class Shell(cmd.Cmd):
                 if key == "fileExtention":
                     if val[0] != ".":
                         val = "." + val
-                self._config.progConfig[key] = val
-                if val.isnumeric():
-                    val = int(val)
-                    self._config.progConfig[key] = val
-                elif val.replace(".","").isnumeric():
-                    val = float(val)
-                    self._config.progConfig[key] = val
-                else:
-                    self._config.progConfig[key] = val
+                self._config.progConfig[key] = strToValue(val)
                 self._config.saveConfig()
             else:
                 eprint("Key not found")
@@ -187,6 +210,7 @@ class Shell(cmd.Cmd):
 
     def do_delete(self, args):
         """delete: delete a subject."""
+        args = shlex.split(args)
         if len(args) == 0:
             subject = self.conversation_parameters["subject"]
         else:
@@ -197,6 +221,7 @@ class Shell(cmd.Cmd):
             eprint("Subject not found")
 
     def do_edit(self, args):
+        args = shlex.split(args)
         """edit: edit a subject."""
         if len(args) == 0:
             subject = self.conversation_parameters["subject"]
@@ -215,13 +240,17 @@ class Shell(cmd.Cmd):
             self._config.printConfig()
             return
         elif len(args) == 1:
-            if args[1] == "save":
+            if args[0] == "save":
                 self._config.saveConfig()
                 return
             eprint("No value provided.")
             return
         elif len(args) == 2:
-            self._config.updateParameter(args[0], args[1])
+            value = strToValue(args[1])
+            if args[0] in self._config.progConfig:
+                self._config.progConfig[args[0]] = value
+            elif args[0] in self._config.conversation_parameters:
+                self._config.updateParameter(args[0], value)
         else:
             eprint("Unrecognized parameter.")
         if not clean:
@@ -232,15 +261,43 @@ class Shell(cmd.Cmd):
     def do_submit(self, args):
         """submit: submit a subject."""
         if self._config.has.get("license", False):
-            response = self._config.chat.submitDialog(self.conversation_parameters["subject"], self.conversation_parameters["scenario"])
-            if response:
-                    text = Text(response)
-                    text.stylize("bold magenta")
-                    console.print(text)
-                    """save to file"""
-                    with open(os.path.join(self._config.conversations_path, self.conversation_parameters["subject"] + self._config.fileExtention), "a") as f:
-                        f.write(f"{self._config.progConfig['aiPrompt']}{response}\n")
-        else: 
+            with console.status("waiting for response ...", spinner="dots"):
+                response = self._config.chat.submitDialog(self.conversation_parameters["subject"], self.conversation_parameters["scenario"])
+                if response:
+                        text = Text(response)
+                        text.stylize("bold magenta")
+                        console.print(text)
+                        """save to file"""
+                        with open(os.path.join(self._config.conversations_path, self.conversation_parameters["subject"] + self._config.fileExtention), "a") as f:
+                            if self.conversation_parameters.get("execute", False):
+                                editPrompt = click.prompt(f"{response}\nedit command? [y/n]", type=click.Choice(["y", "n"]), default="n")
+                                if editPrompt == "y":
+                                    edited = click.edit(response)
+                                    if edited:
+                                        response = edited
+                                doExec = click.prompt(f"{response}\nExecute command? [y/n]", type=click.Choice(["y", "n"]), default="y")
+                                """execute the command in the terminal and edit the response before saving it."""
+                                if doExec == "y":
+                                    result  = subprocess.run(response, stdout=subprocess.PIPE, shell=True, stderr=subprocess.STDOUT)
+                                    result = result.stdout.decode("utf-8")
+                                    print(result)
+                                    saveOutput = click.prompt(f"save output? [Y/e/n]", type=click.Choice(["y", "e", "n"]), default="y")
+                                    if saveOutput == "e":
+                                        edited = click.edit(result)
+                                        if edited:
+                                            result = edited
+                                    if saveOutput != "n":
+                                        f.write(f"{self._config.progConfig['aiPrompt']}{response}")
+                                        f.write("\n")
+                                        f.write(self._config.progConfig["userPrompt"] + str(result))
+                                        f.write("\n")
+                                else:
+                                    f.write(f"{self._config.progConfig['aiPrompt']}{response}")
+                                    f.write("\n")
+                            else:
+                                f.write(f"{self._config.progConfig['aiPrompt']}{response}")
+                                f.write("\n")
+                                        
             self._config.chat.loadLicense()
         return
 
@@ -260,24 +317,176 @@ class Shell(cmd.Cmd):
             self._config.chat.loadLicense()
             return
         if self._config.has.get("license", False):
+            response = None
             with console.status("waiting for response ...", spinner="dots"):
                 response = self._config.chat.query(self.conversation_parameters["subject"], self.conversation_parameters["scenario"], enquiry)
-                if response:
-                    text = Text(response)
-                    text.stylize("bold magenta")
-                    console.print(text)
-                    """save to file"""
-                    with open(os.path.join(self._config.conversations_path, self.conversation_parameters["subject"] + self._config.fileExtention), "a") as f:
-                        f.write(enquiry + "\n" + self._config.progConfig.get("aiPrompt", " AI: ") + response + "\n")
-                        
+            if response:
+                text = Text(response)
+                text.stylize("bold magenta")
+                console.print(f"{text}\n")
+                self.lastResponse = text
+                """save to file"""
+                with open(os.path.join(self._config.conversations_path, self.conversation_parameters["subject"] + self._config.fileExtention), "a") as f:
+                        if self.conversation_parameters.get("execute", False):
+                                editPrompt = click.prompt(f"edit command? [y/n]", type=click.Choice(["y", "n"]), default="n")
+                                if editPrompt == "y":
+                                    edited = click.edit(response)
+                                    if edited:
+                                        response = edited
+                                doExec = click.prompt(f"{response}\nExecute command? [y/n]", type=click.Choice(["y", "n"]), default="y")
+                                """execute the command in the terminal and edit the response before saving it."""
+                                if doExec == "y":
+                                    result  = subprocess.run(response, stdout=subprocess.PIPE, shell=True, stderr=subprocess.STDOUT)
+                                    result = result.stdout.decode("utf-8")
+                                    print(result)
+                                    saveOutput = click.prompt(f"save output? [Y/e/n]", type=click.Choice(["y", "e", "n"]), default="y")
+                                    if saveOutput == "e":
+                                        edited = click.edit(result)
+                                        if edited:
+                                            result = edited
+                                    if saveOutput != "n":
+                                        f.write(self._config.progConfig["userPrompt"] + str(enquiry))
+                                        f.write("\n")
+                                        f.write(f"{self._config.progConfig['aiPrompt']}{response}")
+                                        f.write("\n")
+                                        f.write(self._config.progConfig["userPrompt"] + str(result))
+                                        f.write("\n")
 
-    def complete_query(self,text, line, begidx, endidx):
+                                else:
+                                    f.write(self._config.progConfig["userPrompt"] + str(enquiry))
+                                    f.write("\n")
+                                    f.write(f"{self._config.progConfig['aiPrompt']}{response}")
+                                    f.write("\n")
+                        else:
+                            f.write(self._config.progConfig["userPrompt"] + str(enquiry))
+                            f.write("\n")
+                            f.write(f"{self._config.progConfig['aiPrompt']}{response}")
+                            f.write("\n")
+                            
+    def complete_recap(self,text, line, begidx, endidx):
         """complete_query: complete the query command."""
         if not text:
-            completions = self._config.subjects[:]
+            completions = self._config.get_list()
         else:
             completions = [ f
-                            for f in self._config.subjects
+                            for f in self._config.get_list()
+                            if f.startswith(text)
+                            ]
+        return completions
+
+    def complete_edit(self,text, line, begidx, endidx):
+        """complete_query: complete the query command."""
+        if not text:
+            completions = self._config.get_list()
+        else:
+            completions = [ f
+                            for f in self._config.get_list()
+                            if f.startswith(text)
+                            ]
+        return completions
+
+    def complete_man(self,text, line, begidx, endidx):
+        """complete_query: complete the query command."""
+        if not text:
+            completions = list(self.commands.keys())
+        else:
+            completions = [ f
+                            for f in self.commands.keys()
+                            if f.startswith(text)
+                            ]
+        return completions
+
+  
+    def complete_config(self,text, line, begidx, endidx):
+        """complete_query: complete the query command."""
+        if not text:
+            completions = list(self._config.progConfig.keys())
+        else:
+            completions = [ f
+                            for f in self._config.progConfig.keys()
+                            if f.startswith(text)
+                            ]
+        return completions
+
+    def complete_show(self,text, line, begidx, endidx):
+        """complete_query: complete the query command."""
+        args = shlex.split(line)
+        completions = None
+        # print(f"{args}\n")
+        if len(args) < 2:
+            if not text:
+                completions = list(["config", "scenarios", "subject", "subjects"] )
+            else:
+                completions = [ f
+                                for f in list(["config", "scenarios", "subject", "subjects"] )
+                                if f.startswith(args[-1])
+                                ]
+        elif len(args) == 2:
+            if args[1] == "scenario":
+                completions = list(self._config.scenarios.keys())
+            else:
+                completions = [ f
+                                for f in list(["config", "scenarios", "scenario", "subjects"] )
+                                if f.startswith(args[-1])
+                                ]
+        elif len(args) == 3:
+            completions = [ f
+                            for f in list(self._config.scenarios.keys())
+                            if f.startswith(args[2])
+                            ]
+        return completions
+
+    def complete_set(self,text, line, begidx, endidx):
+        """complete_query: complete the query command."""
+        # print("len: {}, spaces: {}\n".format(len(line), len(line.split(" "))))
+        if  line.rstrip()  == "set":
+            completions = list(self.conversation_parameters.keys())
+            return completions
+        elif len(line.split(" ")) == 2:
+            completions = [ f
+                            for f in self.conversation_parameters.keys()
+                            if f.startswith(text)
+                            ]
+            return completions
+        elif len(line.split(" ")) >  2:
+            completions = line.rstrip().split(" ")[1]
+            if completions == "scenario":
+                completions = [
+                    f
+                    for f in list(self._config.scenarios.keys())
+                    if f.startswith(text) 
+                ]
+                return completions
+            elif completions == "subject":
+                completions = [
+                    f
+                    for f in list(self._config.get_list())
+                    if f.startswith(text) 
+                ]
+                return completions
+            elif completions == "model":
+                completions = [
+                    f
+                    for f in list(self._config.chat.listModels())
+                    if f.startswith(text) 
+                ]
+                return completions
+            elif completions == "defaulCommand":
+                completion =  [
+                    f
+                    for f in list(self.commands.keys())
+                    if f.startswith(text)
+                ]
+                return completions
+
+
+    def complete_delete(self,text, line, begidx, endidx):
+        """complete_query: complete the query command."""
+        if not text:
+            completions = self._config.get_list()
+        else:
+            completions = [ f
+                            for f in self._config.get_list()
                             if f.startswith(text)
                             ]
         return completions
@@ -297,7 +506,7 @@ class Shell(cmd.Cmd):
                 return
             elif args[0] == "scenarios":
                 print("Current scenarios:")
-                for scenario in self._config.scenarios.keys():
+                for scenario in sorted(self._config.scenarios.keys()):
                     print(scenario)
                 return
             elif args[0] == "subjects":
@@ -314,27 +523,28 @@ class Shell(cmd.Cmd):
                     self._config.chat.loadLicense()
                 return
             else:
-                eprint("Unrecognized parameter.")
+                if self.conversation_parameters.get("defaultCommand", "") == "query":
+                    self.do_query(arg)
+                else:
+                    eprint("Unrecognized parameter.")
                 return
         elif len(args) == 2:
-            if args[0] == "subject":
-                subject = sanitizeName(args[1])
-                eprint("Show the conversation inside a subject.")
-                if os.path.isfile(os.path.join(self._config.settingsPath, 'conversations', subject + self._config.fileExtention)):
-                    with open(os.path.join(self._config.settingsPath, 'conversations', subject + self._config.fileExtention), 'r') as f:
-                        print(f.read())
-                        return
-            else:
-                eprint("Unrecognized parameter.")
-                return
+            if args[0] == "scenario":
+                scenario = sanitizeName(args[1])
+                scenario = self._config.scenarios.get(scenario)
+                if scenario is not None:
+                    print("Greetings:")
+                    print(scenario.get("greetings", ""))
+                    for prompt in scenario["conversation"]:
+                        print("{} {}".format(scenario.get(prompt["user"], self._config.progConfig.get(prompt["user"])), prompt["prompt"]))
         else:
-            eprint("Unrecognized parameter.")
-            return
+                if self.conversation_parameters.get("defaultCommand", "") == "query":
+                    self.do_query(arg)
+                    return
+                else:
+                    eprint("Unrecognized parameter.")
+                    return
             
-        """config: configure the model."""
-        print("configuring the model...")
-        print("TODO: implement the config function.")
-
     def saveSession(self):
         """EOF: exit the shell."""
         """ save the current parameters in a toml file to be loaded in the next session"""
@@ -384,7 +594,13 @@ class Shell(cmd.Cmd):
         pass
 
     def default(self, line):
-        if line.startswith("!"):
+        if line == "!!":
+            # take the last response from AI and esecute it. This is meanly useful when using the sharedTermninal scenario. We want to capture the output and feed it into the conversation as the user prompt.
+            if self.lastResponse:
+                self.do_exec(self.lastResponse)
+
+            pass
+        elif line.startswith("!"):
             self.do_exec(line[1:])
         elif self.conversation_parameters.get("defaultCommand","query") == "query":
 
