@@ -18,4 +18,15 @@ def load_json(file):
         try:
             return json.load(f)
         except:
-            return dict()
\ No newline at end of file
+            return dict()
+
+def strToValue(val):
+    if val == "true":
+        val = True
+    elif val == "false":
+        val = False
+    elif val.isnumeric():
+        val = int(val)
+    elif val.replace(".","",1).isnumeric():
+        val = float(val)
+    return val
\ No newline at end of file
